﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            //WebClient wc = new WebClient();

            // 동기 방식
            //string text = wc.DownloadString("http://www.microsoft.com");
            //Console.WriteLine(text);

            // 비동기 방식
            //wc.DownloadStringCompleted += wc_DownloadStringCompleted;
            //wc.DownloadStringAsync(new Uri("http://www.microsoft.com"));

            // async/await을 활용한 비동기 방식
            AwaitDownloadString();

            TcpListener listener = new TcpListener(IPAddress.Any, 11200);
            listener.Start();

            while (true)
            {
                var client = listener.AcceptTcpClient();
                // async/await을 활용한 Socket 비동기 처리
                ProcessTcpClient(client);
            }
        }

        private static async void ProcessTcpClient(TcpClient client)
        {
            NetworkStream ns = client.GetStream();

            byte[] buffer = new byte[1024];
            int received = await ns.ReadAsync(buffer, 0, buffer.Length);

            Thread.Sleep(5000);

            string txt = Encoding.UTF8.GetString(buffer, 0, received);

            byte[] sendBuffer = Encoding.UTF8.GetBytes("Hello: " + txt);
            await ns.WriteAsync(sendBuffer, 0, sendBuffer.Length);
            ns.Close();
        }

        private static async void AwaitDownloadString()
        {
            WebClient wc = new WebClient();
            string text = await wc.DownloadStringTaskAsync("http://www.microsoft.com");
            Console.WriteLine(text);
        }

        static void wc_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            Console.WriteLine(e.Result);
        }
    }
}
